@@ -5,11 +5,11 @@ from collections import namedtuple
 import sqlalchemy
 from sqlalchemy import inspect, orm, and_, or_
 from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
-from sqlalchemy.orm.exc import UnmappedClassError
+import six
+
+from alchy import query, events
+from alchy.utils import classproperty, is_sequence, has_primary_key, camelcase_to_underscore
 
-import query
-import events
-from utils import classproperty, is_sequence, has_primary_key, camelcase_to_underscore
 
 class ModelMeta(DeclarativeMeta):
     def __new__(cls, name, bases, dct):
@@ -61,10 +61,8 @@ class ModelBase(object):
     def __init__(self, *args, **kargs):
         self.update(*args, **kargs)
 
-    def __repr__(self):
-        '''
-        Default representation of model
-        '''
+    def __repr__(self):  # pragma: no cover
+        '''Default representation of model'''
         values = ', '.join(['{0}={1}'.format(c, repr(getattr(self, c))) for c in self.columns])
         return '<{0}({1})>'.format(self.__class__.__name__, values)
 
@@ -78,7 +76,7 @@ class ModelBase(object):
         updatable_fields = self.strict_update_fields if strict else data.keys()
         relationships = self.relationships
 
-        for k,v in data.iteritems():
+        for k, v in six.iteritems(data):
             if hasattr(self, k) and k in updatable_fields:
                 # consider v a dict if any of its elements are a dict
                 v_is_dict = any([isinstance(_v, dict) for _v in v]) if is_sequence(v) else isinstance(v, dict)
@@ -113,9 +111,9 @@ class ModelBase(object):
         d = {}
         descriptors = self.descriptors
 
-        for field, value in self.__dict__.iteritems():
+        for field, value in six.iteritems(self.__dict__):
             if field not in descriptors:
-                # skip sqlalchemy properties
+                # skip non-descriptors
                 continue
 
             if hasattr(value, 'to_dict'):
@@ -136,7 +134,7 @@ class ModelBase(object):
 
     def __iter__(self):
         '''Implement __iter__ so model can be converted to dict via dict(model)'''
-        return self.to_dict().iteritems()
+        return six.iteritems(self.to_dict())
 
     @property
     def strict_update_fields(self):
@@ -156,7 +154,7 @@ class ModelBase(object):
     def get_search(cls, search_dict, filter_fns):
         filters = []
 
-        for key, filter_fn in [(k,v) for k,v in filter_fns.iteritems() if k in search_dict]:
+        for key, filter_fn in [(k, v) for k, v in six.iteritems(filter_fns) if k in search_dict]:
             filters.append(filter_fn(search_dict[key]))
 
         return filters
@@ -186,7 +184,7 @@ class ModelBase(object):
             for term in terms:
                 # create a dict with each `config_search` key and `term` so filters can be applied to each combination
                 # i.e. { config_search_key1: term, config_search_key2: term, ..., config_search_keyN, term }
-                search_dict = dict(zip(fields, [term]*field_count))
+                search_dict = dict(zip(fields, [term] * field_count))
                 term_filters = cls.get_search(search_dict, cls.__simple_search__)
 
                 if term_filters:
@@ -210,22 +208,22 @@ class ModelBase(object):
         return orm.object_session(self)
 
     def flush(self, *args, **kargs):
-        return self.session.flush([self], *args, **kargs)
+        self.session.flush([self], *args, **kargs)
 
     def save(self, *args, **kargs):
-        return self.session.add(self, *args, **kargs)
+        self.session.add(self, *args, **kargs)
 
     def delete(self, *args, **kargs):
         return self.session.delete(self, *args, **kargs)
 
     def expire(self, *args, **kargs):
-        return self.session.expire(self, *args, **kargs)
+        self.session.expire(self, *args, **kargs)
 
     def refresh(self, *args, **kargs):
-        return self.session.refresh(self, *args, **kargs)
+        self.session.refresh(self, *args, **kargs)
 
     def expunge(self, *args, **kargs):
-        return self.session.expunge(self, *args, **kargs)
+        self.session.expunge(self, *args, **kargs)
 
     @classmethod
     def get(cls, *args, **kargs):
@@ -242,6 +240,16 @@ class ModelBase(object):
     # inspect based methods/properties
     ##
 
+    @classproperty
+    def primary_key(cls):
+        '''Return primary key as either single column (one primary key) or tuple otherwise.'''
+        primary = inspect(cls).primary_key
+
+        if len(primary) == 1:
+            primary = primary[0]
+
+        return primary
+
     @classproperty
     def attrs(cls):
         '''Return ORM attributes'''
@@ -250,48 +258,50 @@ class ModelBase(object):
     @classproperty
     def descriptors(cls):
         '''Return all ORM descriptors'''
-        return [d for d in inspect(cls).mapper.all_orm_descriptors.keys() if not d.startswith('__')]
+        return [d for d in inspect(cls).all_orm_descriptors.keys() if not d.startswith('__')]
 
     @classproperty
     def relationships(cls):
         '''Return ORM relationships'''
-        return inspect(cls).mapper.relationships.keys()
+        return inspect(cls).relationships.keys()
 
     @classproperty
     def column_attrs(cls):
         '''Return table columns as list of class attributes at the class level'''
-        return inspect(cls).mapper.column_attrs
+        return inspect(cls).column_attrs
 
     @classproperty
     def columns(cls):
         '''Return table columns'''
-        return inspect(cls).mapper.columns.keys()
+        return inspect(cls).columns.keys()
+
 
 class QueryProperty(object):
     def __init__(self, session):
         self.session = session
 
     def __get__(self, model, Model):
-        try:
-            mapper = orm.class_mapper(Model)
-            if mapper:
-                if not getattr(Model, 'query_class', None):
-                    Model.query_class = query.Query
+        mapper = orm.class_mapper(Model)
+        if mapper:
+            if not getattr(Model, 'query_class', None):
+                Model.query_class = query.Query
 
-                q = Model.query_class(mapper, session=self.session())
-                q.__model__ = Model
+            q = Model.query_class(mapper, session=self.session())
+            q.__model__ = Model
 
-                return q
-        except UnmappedClassError:
-            return None
+            return q
+
+
+def make_declarative_base(session=None, query_property=None, Model=None, Base=None):
+    if Model is None:
+        Base = Base or ModelBase
+        Model = declarative_base(cls=Base, constructor=Base.__init__, metaclass=ModelMeta)
 
-def make_declarative_base(session=None, query_property=None, Model=None):
-    Model = Model or declarative_base(cls=ModelBase, constructor=ModelBase.__init__, metaclass=ModelMeta)
     extend_declarative_base(Model, session, query_property)
     return Model
 
+
 def extend_declarative_base(Model, session=None, query_property=None):
     # attach query attribute to Model if `session` object passed in
     if session:
         Model.query = query_property(session) if query_property else QueryProperty(session)
-
